# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from __future__ import absolute_import, division, print_function

from locker import ls_resources


OBJECT_CLASSES = {
    # data structures
    # ls_resources.ListObject.OBJECT_NAME: ls_resources.ListObject,
    # business objects
    ls_resources.Secret.OBJECT_NAME: ls_resources.Secret,
    ls_resources.Environment.OBJECT_NAME: ls_resources.Environment
}
