from __future__ import absolute_import, division, print_function

from locker.ls_resources.abstract.api_resource import APIResource
from locker.ls_resources.abstract.singleton_api_resource import (
    SingletonAPIResource,
)
from locker.ls_resources.abstract.createable_api_resource import (
    CreateableAPIResource,
)
from locker.ls_resources.abstract.updateable_api_resource import (
    UpdateableAPIResource,
)
from locker.ls_resources.abstract.deletable_api_resource import (
    DeletableAPIResource,
)
from locker.ls_resources.abstract.listable_api_resource import (
    ListableAPIResource,
)

from locker.ls_resources.abstract.detailable_api_resource import (
    DetailableAPIResource,
)
