# -*- coding: utf-8 -*-


"""Meta data of locker-sdk-python"""


__version__ = '1.0.5'
__author__ = 'CyStack Corporation'
__copyright__ = 'Copyright 2023, CyStack Corporation'
__license__ = ''

__all__ = (
    '__version__'
)
